@@ -18,6 +18,7 @@
 #include "xtensor_forward.hpp"
 #include "xslice.hpp"
 #include "xstrides.hpp"
+#include "xtensor_config.hpp"
 #include "xutils.hpp"
 
 namespace xt
@@ -42,16 +43,15 @@ namespace xt
 
             using iterator = decltype(std::declval<std::remove_reference_t<CT>>().template begin<L>());
             using const_iterator = decltype(std::declval<std::decay_t<CT>>().template cbegin<L>());
+            using reverse_iterator = decltype(std::declval<std::remove_reference_t<CT>>().template rbegin<L>());
+            using const_reverse_iterator = decltype(std::declval<std::decay_t<CT>>().template crbegin<L>());
 
             explicit flat_expression_adaptor(CT* e);
 
             template <class FST>
             flat_expression_adaptor(CT* e, FST&& strides);
 
-            void update_pointer(CT* ptr) const
-            {
-                m_e = ptr;
-            }
+            void update_pointer(CT* ptr) const;
 
             size_type size() const;
             reference operator[](size_type idx);
@@ -68,10 +68,13 @@ namespace xt
 
             mutable CT* m_e;
             inner_strides_type m_strides;
-            mutable index_type m_index;
+            thread_local static index_type m_index;
             size_type m_size;
         };
 
+        template <class CT, layout_type L>
+        thread_local typename flat_expression_adaptor<CT, L>::index_type flat_expression_adaptor<CT, L>::m_index;
+
         template <class T>
         struct is_flat_expression_adaptor : std::false_type
         {
@@ -141,6 +144,7 @@ namespace xt
         const inner_strides_type& strides() const noexcept;
         const inner_backstrides_type& backstrides() const noexcept;
         layout_type layout() const noexcept;
+        bool is_contiguous() const noexcept;
         using base_type::shape;
 
         reference operator()();
@@ -401,6 +405,13 @@ namespace xt
     {
         return m_layout;
     }
+
+    template <class D>
+    inline bool xstrided_view_base<D>::is_contiguous() const noexcept
+    {
+        return m_layout != layout_type::dynamic && m_e.is_contiguous();
+    }
+
     //@}
 
     /**
@@ -682,6 +693,12 @@ namespace xt
             m_size = m_e->size();
         }
 
+        template <class CT, layout_type L>
+        inline void flat_expression_adaptor<CT, L>::update_pointer(CT* ptr) const
+        {
+            m_e = ptr;
+        }
+        
         template <class CT, layout_type L>
         inline auto flat_expression_adaptor<CT, L>::size() const -> size_type
         {
@@ -815,7 +832,7 @@ namespace xt
                     {
                         if (has_ellipsis == true)
                         {
-                            throw std::runtime_error("Ellipsis can only appear once.");
+                            XTENSOR_THROW(std::runtime_error, "Ellipsis can only appear once.");
                         }
                         has_ellipsis = true;
                     }
@@ -827,7 +844,7 @@ namespace xt
 
                 if (dimension_check < 0)
                 {
-                    throw std::runtime_error("Too many slices for view.");
+                    XTENSOR_THROW(std::runtime_error, "Too many slices for view.");
                 }
 
                 if (has_ellipsis)
